/* -- automatically generated - do not change -- */
/* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv */

#ifndef _ELG_TRC_H
#define _ELG_TRC_H

/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  Copyright (c) 1998-2009                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef _ESD_DEF_H
#define _ESD_DEF_H

#ifdef __cplusplus
#   define EXTERN extern "C" 
#else
#   define EXTERN extern 
#endif

#include "elg_defs.h"


/*
 *-----------------------------------------------------------------------------
 * Definition records
 *-----------------------------------------------------------------------------
 */

/* 
   This function can be used to register a source file. Generates an
   ELG_FILE record as well as appropriate ELG_STRING and ELG_STRING_CNT records.
   The file identifier is assigned automatically and returned.  
*/

EXTERN elg_ui4  esd_def_file       ( const char* fname );

/* 
   This function can be used to register a source region. Generates an
   ELG_REGION record as well as appropriate ELG_STRING and ELG_STRING_CNT records.
   The file identifier fid has to be defined in advance.
   The region identifier is assigned automatically and returned.
*/

EXTERN elg_ui4  esd_def_region     ( const char* rname,
				     elg_ui4 fid,
				     elg_ui4 begln,
				     elg_ui4 endln,
				     const char* rdesc,
				     elg_ui1 rtype );

/* 
   This function can be used to register a call site. Generates an
   ELG_CALL_SITE record. The file identifier fid as well as the identifiers
   of the regions to be left and entered have to be defined in advance. 
   The call-site identifier is assigned automatically and returned.
*/

EXTERN elg_ui4  esd_def_csite     ( elg_ui4 fid,
				    elg_ui4 lno,
				    elg_ui4 erid, 
				    elg_ui4 lrid );
/* 
   This function can be used to register a call path.  Generates an
   ELG_CALL_PATH record.  The parent call-path identifier ppid as well as
   the identifier of the extension region rid have to be defined in advance. 
   The order parameter is used to organise otherwise identical call-paths
   (e.g., from the same process but with different return addresses, or
   based on first instance timestamp for different processes.)
   The call-path identifier is assigned automatically and returned.
*/

EXTERN elg_ui4  esd_def_callpath  ( elg_ui4 rid,
				    elg_ui4 ppid,
				    elg_ui8 order );

/* 
   Defines an MPI communicator.  Generates an ELG_MPI_COMM record.
*/

EXTERN void esd_def_mpi_comm       ( elg_ui4 cid,
				     elg_ui4 grpc,
				     elg_ui1 grpv[] );

/* 
   Defines an MPI window.  Generates an ELG_MPI_WIN record.
*/

EXTERN void esd_def_mpi_win        ( elg_ui4 wid, elg_ui4 cid);


/*
  This function can be used to register a cartesian topology. Generates an
  ELG_CART_TOPOLOGY record. The number of cartesian dimensions as well as 
  the arrays storing information about the number of processes in each 
  dimension and periodicity in each dimension have to be defined in advance.
*/

EXTERN void  esd_def_cart           ( elg_ui4 topid,
				      elg_ui4 cid,
				      elg_ui1 ndims,
				      elg_ui4 dimv[],
				      elg_ui1 periodv[] );

/* 
   Generates an ELG_CART_COORDS record 
*/

EXTERN void esd_def_coords          ( elg_ui4 topid,
				      elg_ui1 ndims,
				      elg_ui4 coordv[] );

/*
  The machine definition is determined internally.
*/

EXTERN void esd_def_machine         ( elg_ui4 node_id, elg_ui4 proc_id,
                                      elg_ui4 num_procs, elg_ui4 num_thrds );

/*
  Process/thread location within machine.
*/

EXTERN void esd_def_location        ( elg_ui4 mach_id, 
                                      elg_ui4 node_id,
                                      elg_ui4 proc_id,
                                      elg_ui4 thrd_id );

/*
  Metric definitions are internally determined.
*/

EXTERN void esd_def_metrics         ( );

#endif

/*
  Record of event types.
*/

EXTERN void esd_def_event_types     ( elg_ui4 ntypes,
				      elg_ui1 typev[] );

/*
  Record(s) of event type counts.
*/

EXTERN void esd_def_event_counts    ( elg_ui4 ntypes,
				      elg_ui4 countv[] );

/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  Copyright (c) 1998-2009                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef _ELG_TRC_H_GEN
#define _ELG_TRC_H_GEN

#ifdef __cplusplus
#   define EXTERN extern "C" 
#else
#   define EXTERN extern 
#endif

#include "elg_defs.h"

/*
 *-----------------------------------------------------------------------------
 * Event records
 *-----------------------------------------------------------------------------
 */

/* -- automatically generated -- do not change ------------------------------ */
/* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv */

/* -- Region -- */

EXTERN void elg_enter(elg_ui4 rid);

EXTERN void elg_enter_cs(elg_ui4 csid);

EXTERN void elg_exit();

/* -- MPI-1 -- */

EXTERN void elg_mpi_send(elg_ui4 dpid, elg_ui4 cid, elg_ui4 tag, elg_ui4 sent);

EXTERN void elg_mpi_recv(elg_ui4 spid, elg_ui4 cid, elg_ui4 tag, elg_ui4 recvd);

EXTERN void elg_mpi_collexit(elg_ui4 rid, elg_ui4 rpid, elg_ui4 cid,
       elg_ui4 sent, elg_ui4 recvd);

/* -- MPI-2 One-sided Operations -- */

EXTERN void elg_mpi_put_1ts(elg_ui4 dpid, elg_ui4 wid, elg_ui4 rmaid,
       elg_ui4 nbytes);

EXTERN void elg_mpi_put_1te(elg_ui4 spid, elg_ui4 wid, elg_ui4 rmaid);

EXTERN void elg_mpi_put_1te_remote(elg_ui4 dpid, elg_ui4 wid, elg_ui4 rmaid);

EXTERN void elg_mpi_get_1ts(elg_ui4 dpid, elg_ui4 wid, elg_ui4 rmaid,
       elg_ui4 nbytes);

EXTERN void elg_mpi_get_1ts_remote(elg_ui4 dpid, elg_ui4 wid, elg_ui4 rmaid,
       elg_ui4 nbytes);

EXTERN void elg_mpi_get_1te(elg_ui4 spid, elg_ui4 wid, elg_ui4 rmaid);

EXTERN void elg_mpi_winexit(elg_ui4 wid, elg_ui4 cid, elg_ui1 synex);

EXTERN void elg_mpi_wincollexit(elg_ui4 wid);

EXTERN void elg_mpi_win_lock(elg_ui4 lpid, elg_ui4 wid, elg_ui1 ltype);

EXTERN void elg_mpi_win_unlock(elg_ui4 lpid, elg_ui4 wid);

/* -- Generic One-sided Operations -- */

EXTERN void elg_put_1ts(elg_ui4 dpid, elg_ui4 rmaid, elg_ui4 nbytes);

EXTERN void elg_put_1te(elg_ui4 spid, elg_ui4 rmaid);

EXTERN void elg_put_1te_remote(elg_ui4 dpid, elg_ui4 rmaid);

EXTERN void elg_get_1ts(elg_ui4 dpid, elg_ui4 rmaid, elg_ui4 nbytes);

EXTERN void elg_get_1ts_remote(elg_ui4 dpid, elg_ui4 rmaid, elg_ui4 nbytes);

EXTERN void elg_get_1te(elg_ui4 spid, elg_ui4 rmaid);

EXTERN void elg_collexit(elg_ui4 rpid, elg_ui4 cid, elg_ui4 sent,
       elg_ui4 recvd);

EXTERN void elg_alock(elg_ui4 lkid);

EXTERN void elg_rlock(elg_ui4 lkid);

/* -- OpenMP -- */

EXTERN void elg_omp_fork();

EXTERN void elg_omp_join();

EXTERN void elg_omp_alock(elg_ui4 lkid);

EXTERN void elg_omp_rlock(elg_ui4 lkid);

EXTERN void elg_omp_collexit();

/* -- EPILOG Internal -- */

EXTERN void elg_log_off();

EXTERN void elg_log_on();

EXTERN void elg_enter_tracing();

EXTERN void elg_exit_tracing();

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */
/* -- end automatically generated ------------------------------------------- */

#endif
/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
**  KOJAK       http://www.fz-juelich.de/jsc/kojak/                        **
*****************************************************************************
**  Copyright (c) 1998-2009                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  Copyright (c) 2003-2008                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef _ESD_RUN_H
#define _ESD_RUN_H

#ifdef __cplusplus
#   define EXTERN extern "C" 
#else
#   define EXTERN extern 
#endif

#include "elg_defs.h"


/*
 *-----------------------------------------------------------------------------
 *
 *  EPISODE run-time control and measurement management
 *
 *-----------------------------------------------------------------------------
 */

EXTERN int  esd_open               ( );

EXTERN void esd_flush              ( );

EXTERN void esd_close              ( );

/*
  The next two functions have to be called immediately after
  initializing and immediately before finalizing the communication
  middle-ware, e.g. after MPI_Init() and before MPI_Finalize().  
*/

EXTERN int  esd_mpi_init           ( );
EXTERN void esd_mpi_finalize       ( );

/*
  For non-MPI and non-OpenMP programs, the next function has to be
  called after elg_open() to register a unique id (0..num-1) for the
  calling process and the number of processes
*/

EXTERN void esd_init_trc_id        ( int my_id, int num_procs );

#endif

#endif
