/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2009                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#ifndef PEARL_GROUP_H
#define PEARL_GROUP_H


#include <vector>

#include "IdObject.h"


/*-------------------------------------------------------------------------*/
/**
 * @file    Group.h
 * @ingroup PEARL_base
 * @brief   Declaration of the class Group.
 *
 * This header file provides the declaration of the class Group.
 */
/*-------------------------------------------------------------------------*/


namespace pearl
{

//--- Forward declarations --------------------------------------------------

class Process;


/*-------------------------------------------------------------------------*/
/**
 * @class   Group
 * @ingroup PEARL_base
 * @brief   Stores information related to a group of processes.
 *
 * Instances of the Group class provide information about a group of
 * processes with an ordering defined by the numerical identifier (i.e.,
 * the rank) of each process. Such groups are used, e.g., to define MPI
 * communicators (see MpiComm).
 *
 * The numerical identifiers of the individual groups are globally
 * defined and continuously enumerated, i.e., the ID is element of
 * [0,@#process_groups-1].
 */
/*-------------------------------------------------------------------------*/

class Group : public IdObject
{
  public:
    /// Container type for process groups
    typedef std::vector<Process*> process_group;


    /// @name Constructors & destructor
    /// @{

    Group(ident_t id, const process_group& processes);

    /// @}
    /// @name Get group information
    /// @{

    uint32_t num_processes() const;

    Process* get_process(uint32_t rank) const;
    uint32_t get_rank(Process* process) const;

    /// @}


  private:
    /// Processes belonging to this group
    process_group m_processes;
};


}   /* namespace pearl */


#endif   /* !PEARL_GROUP_H */
