/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2009                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#ifndef PEARL_TYPES_H
#define PEARL_TYPES_H


#include <stdint.h>


/*-------------------------------------------------------------------------*/
/**
 * @file    pearl_types.h
 * @ingroup PEARL_base
 * @brief   Definition of data types and constants.
 *
 * This header file provides definitions of PEARL-related data types and
 * constants.
 */
/*-------------------------------------------------------------------------*/


namespace pearl
{

//--- Type definitions ------------------------------------------------------

/// Data type for timestamp values
typedef double timestamp_t;

/// Data type for identifiers
typedef uint32_t ident_t;

/// Data type for line number information
typedef uint32_t line_t;

/// Enumeration defining available event types
enum event_t {
  ANY,              // i.e. any of the event types defined below

  // Generic event types
  FLOW,             // i.e. ENTER, ENTER_CS, EXIT, MPI_COLLEXIT or OMP_COLLEXIT
  ENTER,
  ENTER_CS,
  EXIT,

  // MPI-1 specific event types
  MPI_COLLEXIT,
  MPI_P2P,          // i.e. MPI_SEND or MPI_RECV
  MPI_SEND,
  MPI_RECV,

  // OpenMP specific event types
  OMP_FORK,
  OMP_JOIN,
  OMP_COLLEXIT,
  OMP_LOCK,         // i.e. OMP_ALOCK or OMP_RLOCK
  OMP_ALOCK,
  OMP_RLOCK,

  NUM_EVENT_TYPES   // symbolic name to retrieve the number of event types
};


//--- Constants -------------------------------------------------------------

/// Constant used to identify unused/unknown identifier fields
const ident_t PEARL_NO_ID = 0xFFFFFFFF;

/// Constant used to identify unused/unknown number fields
const line_t  PEARL_NO_NUM = 0;


}   /* namespace pearl */


#endif   /* !PEARL_TYPES_H */
